# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['cjdb',
 'cjdb.model',
 'cjdb.model.sqlalchemy_models',
 'cjdb.modules',
 'cjdb.resources']

package_data = \
{'': ['*']}

install_requires = \
['cjio>=0.8.1,<0.9.0',
 'geoalchemy2>=0.13.1,<0.14.0',
 'numpy>=1.24.2,<2.0.0',
 'psycopg2-binary>=2.9.6,<3.0.0',
 'pyproj>=3.5.0,<4.0.0',
 'requests>=2.28.2,<3.0.0',
 'shapely>=2.0.1,<3.0.0']

entry_points = \
{'console_scripts': ['cjdb = cjdb.cli:cjdb']}

setup_kwargs = {
    'name': 'cjdb',
    'version': '2.0.0',
    'description': 'CJDB is a tool that enables CityJSON integration with a PostgreSQL database',
    'long_description': '# cjdb\n[![MIT badge](https://img.shields.io/pypi/l/cjdb)](LICENSE) &nbsp; [![PyPI](https://img.shields.io/pypi/v/cjdb)](https://pypi.org/project/cjdb)\n\n`cjdb` is a Python-based importer/exporter of [CityJSONL files (CityJSON Lines)](https://www.cityjson.org/cityjsonl/) to and from a PostgreSQL database. \nIt requires the [PostGIS](https://postgis.net/) extension.\n\n\n## Installation\n```bash\npip install cjdb\n```\nIt is recommended to install it in an isolated environment.\n\n\n## Usage\n\nCheck our [docs online](https://cityjson.github.io/cjdb/cjdb.html)  or\n\n```bash\ncjdb --help\n```\n\n## Quickstart\n\nSample CityJSON data can be downloaded from [the 3DBAG download service](https://3dbag.nl/). \nFor example, [download the tile "9-284-556"](https://data.3dbag.nl/cityjson/v20230622/tiles/9/284/556/9-284-556.city.json), within which part of TU Delft is located.\nThen, having downloaded the CityJSON file, you need a combination of [cjio](https://github.com/cityjson/cjio) (external CityJSON processing library) and cjdb to import it to a schema in a database.  Here is a step-by-step guide:\n\n1. Convert CityJSON to CityJSONL\n\n```bash\ncjio --suppress_msg 9-284-556.json export jsonl 9-284-556.jsonl \n```\n\n2. Create a new database called "testcjdb"\n\nIf you installed PostgreSQL you should have the program \'createdb\', so `createdb testcjdb`\n\nAlternatively, you can use PgAdmin, [see how](https://postgis.net/workshops/postgis-intro/creating_db.html).\n\n3. Import CityJSONL to the database in the schema "cjdb"\n```bash\ncjdb import -H localhost -U postgres -d testcjdb -s cjdb  -f 9-284-556.jsonl\n```\n\n**Alternatively steps 1 and 3 in a single command:**\n\n```bash\ncjio --suppress_msg 9-284-556.json export jsonl stdout | cjdb import -H localhost -U postgres -d postgres -s cjdb\n```\n\nThe metadata and the objects can then be found in the tables in the specified schema (`cjdb` in this example).\n\nThe password can be specified in the `PGPASSWORD` environment variable. If not specified, the app will prompt for the password.\n\n4. If you want to export from the database you have two options. You can export the whole database in a CityJSONL file with: \n```bash\ncjdb export -H localhost -U postgres -d testcjdb -s cjdb -o result.jsonl\n```\nor export only part of it, using a select query as input. The select query should return the ids of the objects to be exported:\n\n```bash\ncjdb export -H localhost -U postgres -d testcjdb -s cjdb -o result.jsonl -q "SELECT 1 as id"\n```\n\n## Using docker\nBuild:\n```bash\ndocker build -t cjdb:latest .\n```\n\nRun:\n```bash\ndocker run --rm -it cjdb cjdb --help\n```\n\nTo import some files, the `-v` option is needed to mount our local file directory in the container:\n```bash\ndocker run -v {MYDIRECTORY}:/data --rm -it --network=host cjdb cjdb import -H localhost -U postgres -d postgres -W postgres -f /data/5870_ext.jsonl \n```\n\n## Important Notes\n### Data model\n\nThe `cjdb` importer loads the data in accordance with a [specific data model](cjdb/model/README.md).\n\nFor example SQL queries on the tables see [here](cjdb/model/BASICQUERIES.md)\n\n\n### Indexes\nSome indexes are created when a new schema is created (refer to [Data Model](cjdb/model/README.md)).\n\nIn addition to these indexes, the user can add more indexes on certain  CityObject attributes with the `-x/--attr-index` or the `-px/--partial-attr-index` flags.\nWe recommend these additional indexes for attributes that are frequently queried.\nThe second option uses a partial index with a `not null` condition on the attribute.\nThis saves disk space when indexing an attribute that is not present among all the imported CityObjects.\nThis is often the case with CityJSON, because in a single dataset there can be different object types, with different attributes.\n\n\n### Structuring the database and its schemas\n\nIt is recommended to group together semantically coherent objects, by importing them to the same database schema.\nOne database can have different schemas.\n\nWhile the current data model supports the import of any type of CityJSON objects together (`Building` and `SolitaryVegetationObject`), the data becomes harder to manage for the user. \nExample of this would be having different attributes for the same CityObject type (which should be consistent for data coming from the same source).\n\n\n### Input == CityJSONFeature\nThe importer works only on with [*CityJSONL* files](https://www.cityjson.org/specs/#text-sequences-and-streaming-with-cityjsonfeature), which are CityJSON files decomposed into their *features* (`CityJSONFeature`).\n\nThe easiest way to create these from a CityJSON file is with [cjio](https://github.com/cityjson/cjio), by following [these instructions](https://github.com/cityjson/cjio#stdin-and-stdout).\n\nThe importer supports 3 kinds of input:\n  1. a single CityJSONL file (only those as the output of cjio currently work)\n  1. a directory of CityJSONL files (all files with *jsonl* extensions are located and imported)\n  1. STDIN using the pipe operator: `cat file.jsonl | cjdb ...`\n\n\n### Coordinate Reference Systems\nThe `cjdb` importer does not allow inconsistent CRSs (coordinate reference systems) within the same database schema. For storing data in different CRSs, you have to create different schemas.\n\nThe data needs to be either harmonized beforehand, or the `--transform` flag can be used upon import, to reproject all the geometries to the CRS of the existing schema. \nSpecifying a 2D CRS (instead of a 3D one) will cause the Z-coordinates to remain unchanged.\n\n**Note:** reprojections slow down the import significantly.\n\n**Note:** Source data with missing `"metadata"/"referenceSystem"` cannot be reprojected due to unknown source reference system. \nYou can use the `-I/--srid` flag to set the SRID of the input file. \n\n\n### 3D reprojections\n[`pyproj`](https://pyproj4.github.io/pyproj/stable/) is used for CRS reprojections. \nWhile it supports 3D CRS transformations between different systems, sometimes downloading additional [grids](https://pyproj4.github.io/pyproj/stable/transformation_grids.html) is required. \nThe importer will attempt to download the grids needed for the reprojection, with the following message:\n\n```\nAttempting to download additional grids required for CRS transformation.\nThis can also be done manually, and the files should be put in this folder:\n        {pyproj_directory}\n```\n\nIf that fails, the user will have to download the required grids and put them in the printed `{pyproj_directory}` themselves. \n\n\n### CityJSON Extensions\nIf [CityJSON Extensions](https://www.cityjson.org/extensions/) are present in the imported files, they can be found listed in the `extensions` column in the `cj_metadata` table.\n\nThe [CityJSON specifications](https://www.cityjson.org/specs/#extensions) mention 3 different extendable features, and the `cjdb` importer deals with them as follows:\n\n1. Complex attributes\n\nNo action is taken. These attributes end up in the `attributes` JSONB column.\n\n2. Additional root properties\n\nAdditional root properties are placed in the `extra properties` JSONB column in the `cj_metadata` table.\n\n3. Additional CityObject type\n\nAdditional CityObject types are appended to the list of allowed CityJSON objects.\n\n### CityJSON GeometryTemplate\n[Geometry templates](https://www.cityjson.org/specs/1.1.2/#geometry-templates)\nare resolved for each object geometry, so that the object in the table ends up with its real-world coordinates (instead of vertex references or relative template coordinates).\n\n### Data validation\nThe importer does not validate the structure of the file. It is assumed that the input file is schema-valid ([CityJSON validator](https://validator.cityjson.org/)).\nIt sends out warnings when:\n- CityObject types appear which are defined neither in the main CityJSON specification nor in any of the supplied extensions. \n- the specified target CRS does not have the Z-axis defined\n- the source dataset does not have a CRS defined at all\n\n### Repeated object IDs\nThe importer does not check if an object with a specific ID exists already in the database - every imported object gets and new id. However, at the time of import the importer will detect previously detected files with the same filename. The user can choose to run the import with either the `-g, --ignore-repeated-file` option to import files with the same filename under a different id or `--overwrite` to overwrite *all* previously imported objects with this filename.\n\n\n## Contributors\n\nThis project started as a group project in the [MSc Geomatics at TUDelft](https://geomatics.tudelft.nl/).\nThe original code for the project can be found [here](https://github.com/leoleonsio/cjdb), and the authors were:\n[@cynthiacai56](https://github.com/cynthiacai56), [@LanYan1110](https://github.com/LanYan1110), [@YitongXia](https://github.com/YitongXia), [@Topher2k](https://github.com/Topher2k), [@siebren014](https://github.com/siebren014), [@leoleonsio](https://github.com/leoleonsio)\n\nThis version has been improved and will be maintained by [@GinaStavropoulou](https://github.com/GinaStavropoulou), and [@hugoledoux](https://github.com/hugoledoux).\n\n',
    'author': 'Cynthia Cai',
    'author_email': 'None',
    'maintainer': 'Gina Stavropoulou',
    'maintainer_email': 'g.stavropoulou@tudelft.nl',
    'url': 'https://github.com/tudelft3d/cjdb',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.11,<4.0',
}


setup(**setup_kwargs)
